@@ -1,5 +1,5 @@
 /*
-	SPDX-FileCopyrightText: 2010-2014 Graeme Gott <graeme@gottcode.org>
+	SPDX-FileCopyrightText: 2010-2024 Graeme Gott <graeme@gottcode.org>
 
 	SPDX-License-Identifier: GPL-3.0-or-later
 */
@@ -14,7 +14,6 @@
 #include <QSettings>
 #include <QStandardPaths>
 #include <QUrl>
-#include <QWidget>
 
 //-----------------------------------------------------------------------------
 
@@ -44,7 +43,7 @@ QStringList AddImage::dropEvent(QDropEvent* event)
 	QStringList files;
 	const QList<QUrl> urls = event->mimeData()->urls();
 	for (const QUrl& url : urls) {
-		QString file = url.toLocalFile();
+		const QString file = url.toLocalFile();
 		if (QDir::match(supportedFormats(), file)) {
 			files.append(file);
 		}
@@ -61,11 +60,13 @@ QStringList AddImage::dropEvent(QDropEvent* event)
 
 QStringList AddImage::getOpenFileNames(QWidget* parent)
 {
-	QString dir = QStandardPaths::writableLocation(QStandardPaths::PicturesLocation);
-	dir = QSettings().value("AddImage/Path", dir).toString();
-	QStringList images = QFileDialog::getOpenFileNames(parent, tr("Open Image"), dir, supportedFormats());
+	QSettings settings;
+	const QStringList images = QFileDialog::getOpenFileNames(parent,
+			tr("Open Image"),
+			settings.value("AddImage/Path", QStandardPaths::writableLocation(QStandardPaths::PicturesLocation)).toString(),
+			supportedFormats());
 	if (!images.isEmpty()) {
-		QSettings().setValue("AddImage/Path", QFileInfo(images.last()).absolutePath());
+		settings.setValue("AddImage/Path", QFileInfo(images.last()).absolutePath());
 	}
 	return images;
 }
@@ -80,8 +81,9 @@ QString AddImage::supportedFormats()
 		const QList<QByteArray> imageformats = QImageReader::supportedImageFormats();
 		for (const QByteArray& type : imageformats) {
 			formats.append("*." + type);
+			formats.append("*." + type.toUpper());
 		}
-		string = "Images(" + formats.join(" ") + ")";
+		string = tr("Images") + " (" + formats.join(" ") + ")";
 	}
 	return string;
 }
