@@ -1,5 +1,5 @@
 /*
-	SPDX-FileCopyrightText: 2008-2024 Graeme Gott <graeme@gottcode.org>
+	SPDX-FileCopyrightText: 2008-2025 Graeme Gott <graeme@gottcode.org>
 
 	SPDX-License-Identifier: GPL-3.0-or-later
 */
@@ -54,7 +54,6 @@ Board::Board(QWidget* parent)
 	, m_scrolling(false)
 	, m_selecting(false)
 	, m_finished(false)
-	, m_action_key(0)
 	, m_action_button(Qt::NoButton)
 	, m_random(QRandomGenerator::securelySeeded())
 {
@@ -594,6 +593,15 @@ bool Board::event(QEvent* event)
 #endif
 //-----------------------------------------------------------------------------
 
+void Board::enterEvent(QEnterEvent* event)
+{
+	m_cursor_pos = event->position().toPoint();
+	updateCursor();
+	QWidget::enterEvent(event);
+}
+
+//-----------------------------------------------------------------------------
+
 void Board::paintEvent(QPaintEvent*)
 {
 	QPainter painter(this);
@@ -679,6 +687,7 @@ void Board::keyPressEvent(QKeyEvent* event)
 	// Scroll left
 	case Qt::Key_Left:
 		scroll(QPoint(2 * offset, 0));
+		attachActivePiece();
 		update();
 		updateCursor();
 		break;
@@ -686,6 +695,7 @@ void Board::keyPressEvent(QKeyEvent* event)
 	// Scroll up
 	case Qt::Key_Up:
 		scroll(QPoint(0, 2 * offset));
+		attachActivePiece();
 		update();
 		updateCursor();
 		break;
@@ -693,6 +703,7 @@ void Board::keyPressEvent(QKeyEvent* event)
 	// Scroll right
 	case Qt::Key_Right:
 		scroll(QPoint(-2 * offset, 0));
+		attachActivePiece();
 		update();
 		updateCursor();
 		break;
@@ -700,6 +711,7 @@ void Board::keyPressEvent(QKeyEvent* event)
 	// Scroll down
 	case Qt::Key_Down:
 		scroll(QPoint(0, -2 * offset));
+		attachActivePiece();
 		update();
 		updateCursor();
 		break;
@@ -735,25 +747,13 @@ void Board::keyPressEvent(QKeyEvent* event)
 		break;
 
 	default:
-		if (!event->isAutoRepeat()) {
-			m_action_key = event->key();
-		}
+		break;
 	}
 	QWidget::keyPressEvent(event);
 }
 
 //-----------------------------------------------------------------------------
 
-void Board::keyReleaseEvent(QKeyEvent* event)
-{
-	if (!event->isAutoRepeat()) {
-		m_action_key = 0;
-	}
-	QWidget::keyReleaseEvent(event);
-}
-
-//-----------------------------------------------------------------------------
-
 void Board::mousePressEvent(QMouseEvent* event)
 {
 	if (m_finished || m_action_button != Qt::NoButton) {
@@ -761,10 +761,10 @@ void Board::mousePressEvent(QMouseEvent* event)
 	}
 
 	m_action_button = event->button();
-	if (m_action_button == Qt::MiddleButton || (m_action_button == Qt::LeftButton && m_action_key == Qt::Key_Shift)) {
+	if (m_action_button == Qt::MiddleButton || (m_action_button == Qt::LeftButton && QGuiApplication::keyboardModifiers() == Qt::ShiftModifier)) {
 		startScrolling();
 	} else if (m_action_button == Qt::LeftButton) {
-		m_select_pos = event->pos();
+		m_select_pos = event->position().toPoint();
 	}
 
 	QWidget::mousePressEvent(event);
@@ -781,6 +781,9 @@ void Board::mouseReleaseEvent(QMouseEvent* event)
 	switch (m_action_button) {
 	case Qt::LeftButton:
 		togglePiecesUnderCursor();
+		if (m_scrolling) {
+			stopScrolling();
+		}
 		break;
 
 	case Qt::RightButton:
@@ -803,7 +806,32 @@ void Board::mouseReleaseEvent(QMouseEvent* event)
 
 void Board::mouseMoveEvent(QMouseEvent* event)
 {
-	const QPoint delta = (event->pos() / m_scale) - (m_cursor_pos / m_scale);
+	const QPoint delta = (event->position().toPoint() / m_scale) - (m_cursor_pos / m_scale);
+	m_cursor_pos = event->position().toPoint();
+
+	if (m_action_button == Qt::LeftButton) {
+		const Qt::KeyboardModifiers modifiers = QGuiApplication::keyboardModifiers();
+
+		// Scroll board if only shift is pressed
+		if (!m_scrolling && modifiers == Qt::ShiftModifier) {
+			startScrolling();
+		} else if (m_scrolling && modifiers != Qt::ShiftModifier) {
+			stopScrolling();
+			m_select_pos = m_cursor_pos;
+		}
+
+		// Select pieces if no keyboard modifiers are pressed
+		if (!m_selecting && modifiers == Qt::NoModifier) {
+			m_selecting = QVector2D(m_cursor_pos - m_select_pos).length() >= QApplication::startDragDistance();
+		} else if (m_selecting && modifiers != Qt::NoModifier) {
+			m_selecting = false;
+			for (Piece* piece : std::as_const(m_selected_pieces)) {
+				piece->setSelected(false);
+			}
+			m_pieces += m_selected_pieces;
+			m_selected_pieces.clear();
+		}
+	}
 
 	if (m_scrolling) {
 		scroll(delta);
@@ -815,23 +843,11 @@ void Board::mouseMoveEvent(QMouseEvent* event)
 			piece->moveBy(delta);
 		}
 
-		// Attach neighbors if only one piece is active
-		if (m_active_pieces.count() == 1) {
-			m_active_pieces.first()->attachSolutionNeighbors();
-			updateCompleted();
-		}
-
-		// Handle finishing game
-		if (pieceCount() == 1) {
-			finishGame();
-		}
+		attachActivePiece();
 	}
 
-	if (!m_selecting && m_action_button == Qt::LeftButton && m_action_key == 0) {
-		m_selecting = QVector2D(event->pos() - m_select_pos).length() >= QApplication::startDragDistance();
-	}
 	if (m_selecting) {
-		const QRect rect = QRect(mapPosition(event->pos()), mapPosition(m_select_pos)).normalized();
+		const QRect rect = QRect(mapPosition(m_cursor_pos), mapPosition(m_select_pos)).normalized();
 
 		// Check for pieces that are now selected
 		for (int i = 0; i < m_pieces.count(); ++i) {
@@ -853,16 +869,11 @@ void Board::mouseMoveEvent(QMouseEvent* event)
 			}
 		}
 
-		m_selection = QRect(event->pos(), m_select_pos).normalized();
+		m_selection = QRect(m_cursor_pos, m_select_pos).normalized();
 	}
 
 	update();
-
-	m_cursor_pos = event->pos();
-
-	if (!m_scrolling) {
-		updateCursor();
-	}
+	updateCursor();
 }
 
 //-----------------------------------------------------------------------------
@@ -891,6 +902,7 @@ void Board::wheelEvent(QWheelEvent* event)
 void Board::edgeScroll(int horizontal, int vertical)
 {
 	scroll(QPoint((5 * horizontal) / m_scale, (5 * vertical) / m_scale));
+	attachActivePiece();
 	update();
 }
 
@@ -899,7 +911,7 @@ void Board::edgeScroll(int horizontal, int vertical)
 void Board::startScrolling()
 {
 	m_scrolling = true;
-	setCursor(Qt::SizeAllCursor);
+	updateCursor();
 }
 
 //-----------------------------------------------------------------------------
@@ -923,9 +935,25 @@ void Board::scroll(const QPoint& delta)
 
 //-----------------------------------------------------------------------------
 
+void Board::attachActivePiece()
+{
+	// Attach neighbors if only one piece is active
+	if (m_active_pieces.count() == 1) {
+		m_active_pieces.first()->attachSolutionNeighbors();
+		updateCompleted();
+	}
+
+	// Handle finishing game
+	if (pieceCount() == 1) {
+		finishGame();
+	}
+}
+
+//-----------------------------------------------------------------------------
+
 void Board::togglePiecesUnderCursor() {
-	switch (m_action_key) {
-	case 0:
+	switch (QGuiApplication::keyboardModifiers()) {
+	case Qt::NoModifier:
 		if (!m_selecting) {
 			if (pieceUnderCursor()) {
 				grabPiece();
@@ -937,14 +965,10 @@ void Board::togglePiecesUnderCursor() {
 		}
 		break;
 
-	case Qt::Key_Shift:
-		stopScrolling();
-		break;
-
 #if !defined(Q_OS_MAC)
-	case Qt::Key_Control:
+	case Qt::ControlModifier:
 #else
-	case Qt::Key_Meta:
+	case Qt::MetaModifier:
 #endif
 		rotatePiece();
 		break;
@@ -1139,6 +1163,11 @@ void Board::loadImage()
 
 void Board::updateCursor()
 {
+	if (m_scrolling) {
+		setCursor(Qt::SizeAllCursor);
+		return;
+	}
+
 	int state = 0;
 	if (!m_finished) {
 		state = (pieceUnderCursor() || m_selecting) | (!m_active_pieces.isEmpty() * 2);
