@@ -1,5 +1,5 @@
 /*
-	SPDX-FileCopyrightText: 2008-2019 Graeme Gott <graeme@gottcode.org>
+	SPDX-FileCopyrightText: 2008-2024 Graeme Gott <graeme@gottcode.org>
 
 	SPDX-License-Identifier: GPL-3.0-or-later
 */
@@ -7,10 +7,12 @@
 #include "new_game_tab.h"
 
 #include "add_image.h"
+#include "choose_game_dialog.h"
 #include "image_properties_dialog.h"
 #include "path.h"
 #include "tag_manager.h"
 #include "thumbnail_delegate.h"
+#include "thumbnail_item.h"
 #include "thumbnail_loader.h"
 #include "toolbar_list.h"
 
@@ -21,7 +23,6 @@
 #include <QDialog>
 #include <QDir>
 #include <QFile>
-#include <QFileDialog>
 #include <QFileInfo>
 #include <QGridLayout>
 #include <QImageReader>
@@ -32,7 +33,6 @@
 #include <QScrollBar>
 #include <QSettings>
 #include <QSlider>
-#include <QSplitter>
 #include <QXmlStreamReader>
 
 #include <algorithm>
@@ -43,6 +43,13 @@
 namespace
 {
 
+/**
+ * Calculate the hash of a file.
+ *
+ * @param path the file to hash
+ *
+ * @return the hash of the file contents
+ */
 QString hash(const QString& path)
 {
 	QFile file(path);
@@ -52,19 +59,15 @@ QString hash(const QString& path)
 	return QCryptographicHash::hash(file.readAll(), QCryptographicHash::Sha1).toHex();
 }
 
-enum ItemRoles
-{
-	TagsRole = Qt::UserRole,
-	ImageRole,
-	NameRole
-};
-
+/**
+ * Update the tooltip of a thumbnail item.
+ */
 void updateToolTip(QListWidgetItem* item)
 {
 	QString tip = item->text();
-	QString tags = item->data(TagsRole).toString();
+	const QString tags = item->data(ThumbnailItem::TagsRole).toString();
 	if (!tags.isEmpty()) {
-		tip += "<br><small><i>" + item->data(TagsRole).toString() + "</i></small>";
+		tip += "<br><small><i>" + tags + "</i></small>";
 	}
 	item->setToolTip(tip);
 }
@@ -76,40 +79,39 @@ void updateToolTip(QListWidgetItem* item)
 NewGameTab::NewGameTab(const QStringList& files, QDialog* parent)
 	: QWidget(parent)
 {
-	// Add image filter
-	m_image_tags = new TagManager(this);
-	connect(m_image_tags, &TagManager::filterChanged, this, &NewGameTab::filterImages);
-	connect(m_image_tags, &TagManager::tagsChanged, this, &NewGameTab::updateTagsStrings);
-
 	// Add image selector
 	m_images = new ToolBarList(this);
 	m_images->setViewMode(QListView::IconMode);
+	m_images->setSelectionMode(QListWidget::ExtendedSelection);
 	m_images->setIconSize(QSize(74, 74));
 	m_images->setMinimumSize(460 + m_images->verticalScrollBar()->sizeHint().width(), 230);
 	m_images->setItemDelegate(new ThumbnailDelegate(m_images));
-	connect(m_images, &ToolBarList::currentItemChanged, this, &NewGameTab::imageSelected);
-	connect(m_images, &ToolBarList::itemActivated, this, &NewGameTab::editImageProperties);
+	connect(m_images, &ToolBarList::itemSelectionChanged, this, &NewGameTab::imageSelected);
 
 	// Add image actions
-	QAction* add_action = new QAction(m_images->fetchIcon("list-add"), tr("Add Image"), this);
+	QAction* add_action = new QAction(QIcon::fromTheme("list-add"), tr("Add Image"), this);
 	m_images->addToolBarAction(add_action);
 	connect(add_action, &QAction::triggered, this, &NewGameTab::addImageClicked);
 
-	m_remove_action = new QAction(m_images->fetchIcon("list-remove"), tr("Remove Image"), this);
+	m_remove_action = new QAction(QIcon::fromTheme("list-remove"), tr("Remove Image"), this);
+	m_remove_action->setEnabled(false);
 	m_images->addToolBarAction(m_remove_action);
 	connect(m_remove_action, &QAction::triggered, this, &NewGameTab::removeImage);
 
-	m_tag_action = new QAction(m_images->fetchIcon("image-x-generic"), tr("Image Properties"), this);
+	m_tag_action = new QAction(QIcon::fromTheme("document-properties"), tr("Image Properties"), this);
+	m_tag_action->setEnabled(false);
 	m_images->addToolBarAction(m_tag_action);
 	connect(m_tag_action, &QAction::triggered, this, &NewGameTab::editImageProperties);
 
-	// Add image splitter
-	m_image_contents = new QSplitter(this);
-	m_image_contents->addWidget(m_image_tags);
-	m_image_contents->addWidget(m_images);
-	m_image_contents->setStretchFactor(0, 0);
-	m_image_contents->setStretchFactor(1, 1);
-	m_image_contents->setSizes(QList<int>() << 130 << m_images->minimumWidth());
+	// Add image filter
+	QWidget* spacer = new QWidget(this);
+	spacer->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
+	m_images->addToolBarWidget(spacer);
+
+	m_image_tags = new TagManager(this);
+	connect(m_image_tags, &TagManager::filterChanged, this, &NewGameTab::filterImages);
+	connect(m_image_tags, &TagManager::tagsChanged, this, &NewGameTab::updateTagsStrings);
+	m_images->addToolBarWidget(m_image_tags);
 
 	// Add pieces slider
 	m_slider = new QSlider(Qt::Horizontal, this);
@@ -126,13 +128,14 @@ NewGameTab::NewGameTab(const QStringList& files, QDialog* parent)
 	connect(buttons, &QDialogButtonBox::rejected, parent, &QDialog::reject);
 
 	m_accept_button = buttons->button(QDialogButtonBox::Ok);
+	m_accept_button->setText(ChooseGameDialog::tr("Play Game"));
 	m_accept_button->setEnabled(false);
 
 	// Arrange widgets
 	QGridLayout* layout = new QGridLayout(this);
 	layout->setColumnStretch(1, 1);
 	layout->setRowStretch(0, 1);
-	layout->addWidget(m_image_contents, 0, 0, 1, 2);
+	layout->addWidget(m_images, 0, 0, 1, 2);
 	layout->setRowMinimumHeight(1, 12);
 	layout->addWidget(m_count, 2, 0);
 	layout->addWidget(m_slider, 2, 1);
@@ -148,13 +151,13 @@ NewGameTab::NewGameTab(const QStringList& files, QDialog* parent)
 	m_images->sortItems();
 
 	// Load values
-	QSettings settings;
+	const QSettings settings;
 	QListWidgetItem* item = m_images->item(0);
-	QString image = settings.value("NewGame/Image").toString();
+	const QString image = settings.value("NewGame/Image").toString();
 	if (!image.isEmpty()) {
 		for (int i = m_images->count() - 1; i >= 0; --i) {
 			item = m_images->item(i);
-			if (item->data(ImageRole) == image) {
+			if (item->data(ThumbnailItem::ImageRole) == image) {
 				break;
 			}
 		}
@@ -165,25 +168,25 @@ NewGameTab::NewGameTab(const QStringList& files, QDialog* parent)
 
 	// Add new images
 	addImages(files);
-
-	// Resize contents
-	m_image_contents->restoreState(settings.value("NewGame/SplitterSizes").toByteArray());
 }
 
 //-----------------------------------------------------------------------------
 
 void NewGameTab::addImages(const QStringList& images)
 {
-	int count = images.count();
+	const int count = images.count();
 	if (count == 0) {
 		return;
 	}
 
+	m_images->clearSelection();
+
 	QProgressDialog progress(tr("Copying images..."), tr("Cancel"), 0, count, this);
 	progress.setMinimumDuration(500);
 	progress.setWindowModality(Qt::WindowModal);
 
 	QApplication::setOverrideCursor(Qt::WaitCursor);
+	QCoreApplication::processEvents();
 
 	m_image_tags->clearFilter();
 	for (int i = 0; i < count; i++) {
@@ -203,22 +206,14 @@ void NewGameTab::addImages(const QStringList& images)
 
 //-----------------------------------------------------------------------------
 
-void NewGameTab::hideEvent(QHideEvent* event)
-{
-	QSettings().setValue("NewGame/SplitterSizes", m_image_contents->saveState());
-	QWidget::hideEvent(event);
-}
-
-//-----------------------------------------------------------------------------
-
 void NewGameTab::accept()
 {
-	QListWidgetItem* item = m_images->currentItem();
-	if (!item) {
+	const QList<QListWidgetItem*> items = m_images->selectedItems();
+	if (items.count() != 1) {
 		return;
 	}
 
-	QString image = item->data(ImageRole).toString();
+	const QString image = items.first()->data(ThumbnailItem::ImageRole).toString();
 
 	QSettings settings;
 	settings.setValue("NewGame/Pieces", m_slider->value());
@@ -238,16 +233,20 @@ void NewGameTab::addImageClicked()
 
 void NewGameTab::removeImage()
 {
-	QListWidgetItem* item = m_images->currentItem();
-	if (!item) {
+	// Find selected images
+	const QList<QListWidgetItem*> items = m_images->selectedItems();
+	if (items.isEmpty()) {
 		return;
 	}
-	QString current_image = item->data(ImageRole).toString();
+	QStringList selected_images;
+	for (const QListWidgetItem* item : items) {
+		selected_images.append(item->data(ThumbnailItem::ImageRole).toString());
+	}
 
+	// Find games that would be affected
 	QStringList games;
 
 	QXmlStreamReader xml;
-	QXmlStreamAttributes attributes;
 	const QStringList files = QDir(Path::saves(), "*.xml").entryList(QDir::Files);
 	for (const QString& game : files) {
 		QFile file(Path::save(game));
@@ -260,47 +259,67 @@ void NewGameTab::removeImage()
 		while (!xml.isStartElement()) {
 			xml.readNext();
 		}
-		attributes = xml.attributes();
+		const QXmlStreamAttributes attributes = xml.attributes();
 		if (xml.name() == QLatin1String("tetzle") && attributes.value("version").toString().toUInt() <= 5) {
-			if (attributes.value("image").toString() == current_image) {
+			if (selected_images.contains(attributes.value("image").toString())) {
 				games.append(game);
 			}
 		}
 	}
 
-	QString message;
-	if (games.isEmpty()) {
-		message = tr("Remove selected image?");
-	} else {
-		message = tr("Remove selected image?\n\nThere are saved games using this image that will also be removed.");
+	// Prompt about image removal
+	const int plural = selected_images.count();
+	QMessageBox mbox(this);
+	mbox.setIcon(QMessageBox::Question);
+	mbox.setWindowTitle(tr("Remove Image"));
+	mbox.setText(tr("Remove %n selected image(s)?", nullptr, plural));
+	if (!games.isEmpty()) {
+		mbox.setIcon(QMessageBox::Warning);
+		mbox.setInformativeText(tr("Saved games using these image(s) will be deleted.", nullptr, plural));
+	}
+	mbox.setStandardButtons(QMessageBox::Cancel);
+	mbox.addButton(tr("Remove"), QMessageBox::AcceptRole);
+	mbox.setDefaultButton(QMessageBox::Cancel);
+	if (mbox.exec() == QMessageBox::Cancel) {
+		return;
 	}
-	if (QMessageBox::question(this, tr("Remove Image"), message, QMessageBox::Yes | QMessageBox::No, QMessageBox::No) == QMessageBox::Yes) {
-		QString image_id = current_image.section(".", 0, 0);
 
+	// Remove images
+	QSettings details(Path::image("details"), QSettings::IniFormat);
+	for (const QString& current_image : std::as_const(selected_images)) {
+		// Remove from image details
+		details.remove(current_image);
+
+		// Delete image
 		QFile::remove(Path::image(current_image));
 
-		QDir dir(Path::thumbnails(), image_id + "*");
-		const QStringList thumbs = dir.entryList();
-		for (const QString& thumb : thumbs) {
-			dir.remove(thumb);
-		}
-
-		for (const QString& game : std::as_const(games)) {
-			QFile::remove(Path::save(game));
-		}
-
-		delete item;
+		// Delete thumbnail
+		removeThumbnail(current_image.section(".", 0, 0));
 
+		// Remove from tags
 		m_image_tags->removeImage(current_image);
+	}
 
-		m_accept_button->setEnabled(m_images->count() > 0);
-		if (!m_accept_button->isEnabled()) {
-			m_slider->setMaximum(-1);
-			m_count->clear();
-			QSettings settings;
-			settings.remove("NewGame/Image");
-			settings.remove("NewGame/Pieces");
-		}
+	// Remove saved games depending on images
+	for (const QString& game : std::as_const(games)) {
+		QFile::remove(Path::save(game));
+	}
+
+	// Remove listwidget items
+	for (const QListWidgetItem* item : items) {
+		delete item;
+	}
+
+	// Handle all images being removed
+	if (m_images->count() == 0) {
+		m_accept_button->setEnabled(false);
+
+		m_slider->setMaximum(-1);
+		m_count->clear();
+
+		QSettings settings;
+		settings.remove("NewGame/Image");
+		settings.remove("NewGame/Pieces");
 	}
 }
 
@@ -308,18 +327,19 @@ void NewGameTab::removeImage()
 
 void NewGameTab::editImageProperties()
 {
-	QListWidgetItem* item = m_images->currentItem();
-	if (!item || item->isHidden()) {
+	const QList<QListWidgetItem*> items = m_images->selectedItems();
+	if (items.count() != 1) {
 		return;
 	}
+	QListWidgetItem* item = items.first();
 
-	QString filename = item->data(ImageRole).toString();
+	const QString filename = item->data(ThumbnailItem::ImageRole).toString();
 	ImagePropertiesDialog dialog(item->icon(), item->text(), m_image_tags, filename, window());
 	if (dialog.exec() == QDialog::Accepted) {
 		// Update name
 		item->setText(dialog.name());
-		if (item->text() != item->data(NameRole).toString()) {
-			item->setData(NameRole, item->text());
+		if (item->text() != item->data(ThumbnailItem::NameRole).toString()) {
+			item->setData(ThumbnailItem::NameRole, item->text());
 
 			QSettings details(Path::image("details"), QSettings::IniFormat);
 			details.setValue(filename + "/Name", item->text());
@@ -330,27 +350,41 @@ void NewGameTab::editImageProperties()
 		}
 
 		// Update tags
-		item->setData(TagsRole, m_image_tags->tags(item->data(ImageRole).toString()));
+		item->setData(ThumbnailItem::TagsRole, m_image_tags->tags(filename));
 		updateToolTip(item);
-
 	}
 }
 
 //-----------------------------------------------------------------------------
 
-void NewGameTab::imageSelected(QListWidgetItem* item)
+void NewGameTab::imageSelected()
 {
-	bool enabled = item;
-	m_accept_button->setEnabled(enabled);
-	m_tag_action->setEnabled(enabled);
-	m_remove_action->setEnabled(enabled);
-	if (!enabled) {
+	const QList<QListWidgetItem*> images = m_images->selectedItems();
+
+	// Disable actions if no images are selected
+	if (images.isEmpty()) {
+		m_accept_button->setEnabled(false);
+		m_tag_action->setEnabled(false);
+		m_remove_action->setEnabled(false);
 		return;
 	}
 
+	// Enable playing game or editing image properties if only 1 image is selected
+	const bool enabled = images.count() == 1;
+	m_accept_button->setEnabled(enabled);
+	m_tag_action->setEnabled(enabled);
+
 	// Prevent removing the image of the game currently open
-	QString image = item->data(ImageRole).toString();
-	m_remove_action->setEnabled(QSettings().value("OpenGame/Image").toString() != image);
+	m_remove_action->setEnabled(true);
+	const QString current_image = QSettings().value("OpenGame/Image").toString();
+	for (const QListWidgetItem* item : images) {
+		if (item->data(ThumbnailItem::ImageRole).toString() == current_image) {
+			m_remove_action->setEnabled(false);
+			break;
+		}
+	}
+
+	const QString image = images.last()->data(ThumbnailItem::ImageRole).toString();
 
 	m_image_size = QImageReader(Path::image(image)).size();
 	if (m_image_size.width() > m_image_size.height()) {
@@ -359,8 +393,8 @@ void NewGameTab::imageSelected(QListWidgetItem* item)
 		m_ratio = static_cast<float>(m_image_size.width()) / static_cast<float>(m_image_size.height());
 	}
 
-	int max = std::lround(std::sqrt(250.0f / m_ratio));
-	int min = std::lround(std::sqrt(2.5f / m_ratio));
+	const int max = std::lround(std::sqrt(250.0f / m_ratio));
+	const int min = std::lround(std::sqrt(2.5f / m_ratio));
 	int value = min;
 	if (m_images->count() > 1) {
 		value = std::lround(static_cast<float>(max * m_slider->value()) / static_cast<float>(m_slider->maximum()));
@@ -375,8 +409,8 @@ void NewGameTab::imageSelected(QListWidgetItem* item)
 void NewGameTab::pieceCountChanged(int value)
 {
 	if (m_image_size.isValid()) {
-		int side1 = 4 * value;
-		int side2 = std::max(std::lround(side1 * m_ratio), 1L);
+		const int side1 = 4 * value;
+		const int side2 = std::max(std::lround(side1 * m_ratio), 1L);
 		m_count->setText(tr("%L1 pieces").arg(side1 * side2 / 4));
 	}
 }
@@ -386,47 +420,52 @@ void NewGameTab::pieceCountChanged(int value)
 void NewGameTab::filterImages(const QStringList& filter)
 {
 	// Filter items
-	QListWidgetItem* item;
-	int count = m_images->count();
-	for (int i = 0; i < count; ++i) {
-		item = m_images->item(i);
-		item->setHidden(!filter.contains(item->data(ImageRole).toString()));
-	}
+	m_images->blockSignals(true);
+	for (int i = 0, count = m_images->count(); i < count; ++i) {
+		QListWidgetItem* item = m_images->item(i);
 
-	// Select next item if current item was hidden
-	item = m_images->currentItem();
-	if (!item || item->isHidden()) {
-		int count = m_images->count();
-		for (int i = 0; i < count; ++i) {
-			item = m_images->item(i);
-			if (!item->isHidden()) {
-				item->setSelected(true);
-				m_images->setCurrentItem(item);
-				break;
-			}
+		if (filter.contains(item->data(ThumbnailItem::ImageRole).toString())) {
+			item->setHidden(false);
+		} else {
+			item->setHidden(true);
+			item->setSelected(false);
 		}
 	}
+	m_images->blockSignals(false);
 
 	// Disable tag button if no images are visible
-	item = m_images->currentItem();
-	m_tag_action->setEnabled(item && !item->isHidden());
+	imageSelected();
 }
 
 //-----------------------------------------------------------------------------
 
 void NewGameTab::updateTagsStrings()
 {
-	QListWidgetItem* item;
-	int count = m_images->count();
-	for (int i = 0; i < count; ++i) {
-		item = m_images->item(i);
-		item->setData(TagsRole, m_image_tags->tags(item->data(ImageRole).toString()));
+	for (int i = 0, count = m_images->count(); i < count; ++i) {
+		QListWidgetItem* item = m_images->item(i);
+		item->setData(ThumbnailItem::TagsRole, m_image_tags->tags(item->data(ThumbnailItem::ImageRole).toString()));
 		updateToolTip(item);
 	}
 }
 
 //-----------------------------------------------------------------------------
 
+void NewGameTab::hideEvent(QHideEvent* event)
+{
+	m_accept_button->setDefault(false);
+	QWidget::hideEvent(event);
+}
+
+//-----------------------------------------------------------------------------
+
+void NewGameTab::showEvent(QShowEvent* event)
+{
+	m_accept_button->setDefault(true);
+	QWidget::showEvent(event);
+}
+
+//-----------------------------------------------------------------------------
+
 void NewGameTab::addImage(const QString& image)
 {
 	QImageReader reader(image);
@@ -444,7 +483,7 @@ void NewGameTab::addImage(const QString& image)
 	for (const QString& file : images) {
 		image_id = std::max(image_id, file.section(".", 0, 0).toInt());
 
-		QString key = file + "/SHA1";
+		const QString key = file + "/SHA1";
 		if (!details.contains(key)) {
 			details.setValue(key, hash(Path::image(file)));
 		}
@@ -458,7 +497,7 @@ void NewGameTab::addImage(const QString& image)
 	QListWidgetItem* item = nullptr;
 	if (filename.isEmpty()) {
 		// Find filename
-		QFileInfo info(image);
+		const QFileInfo info(image);
 		filename = QString("%1.%2").arg(image_id).arg(info.suffix().toLower());
 		details.setValue(filename + "/SHA1", image_hash);
 		details.setValue(filename + "/Name", info.completeBaseName());
@@ -469,13 +508,15 @@ void NewGameTab::addImage(const QString& image)
 		if (reader.transformation() == QImageIOHandler::TransformationNone) {
 			QFile::copy(image, Path::image(filename));
 		} else {
-			QImage image = reader.read();
-			image.save(Path::image(filename), "", 100);
+			reader.read().save(Path::image(filename), "", 100);
 		}
+
+		// Remove old thumbnail if it exists
+		removeThumbnail(QString::number(image_id));
 	} else {
 		// Find in list of images
-		for (int i = 0; i < m_images->count(); ++i) {
-			if (m_images->item(i)->data(ImageRole).toString() == filename) {
+		for (int i = 0, count = m_images->count(); i < count; ++i) {
+			if (m_images->item(i)->data(ThumbnailItem::ImageRole).toString() == filename) {
 				item = m_images->item(i);
 				break;
 			}
@@ -487,7 +528,7 @@ void NewGameTab::addImage(const QString& image)
 		m_images->blockSignals(true);
 		item = createItem(filename, details);
 		m_images->blockSignals(false);
-		m_images->setCurrentItem(item);
+		m_images->sortItems();
 	}
 	m_images->setCurrentItem(item);
 	m_images->scrollToItem(item, QAbstractItemView::PositionAtTop);
@@ -498,12 +539,22 @@ void NewGameTab::addImage(const QString& image)
 QListWidgetItem* NewGameTab::createItem(const QString& image, const QSettings& details)
 {
 	const qreal pixelratio = devicePixelRatioF();
-	QListWidgetItem* item = ThumbnailLoader::createItem(Path::image(image), details.value(image + "/Name", tr("Untitled")).toString(), m_images, pixelratio);
-	item->setData(ImageRole, image);
-	item->setData(NameRole, item->text());
-	item->setData(TagsRole, m_image_tags->tags(image));
+	QListWidgetItem* item = ThumbnailLoader::createItem(image, details.value(image + "/Name", tr("Untitled")).toString(), m_images, pixelratio);
+	item->setData(ThumbnailItem::NameRole, item->text());
+	item->setData(ThumbnailItem::TagsRole, m_image_tags->tags(image));
 	updateToolTip(item);
 	return item;
 }
 
 //-----------------------------------------------------------------------------
+
+void NewGameTab::removeThumbnail(const QString& image_id) const
+{
+	QDir dir(Path::thumbnails());
+	const QStringList thumbnails = dir.entryList({ image_id + ".*", image_id + "@*" }, QDir::Files);
+	for (const QString& file : thumbnails) {
+		dir.remove(file);
+	}
+}
+
+//-----------------------------------------------------------------------------
