@@ -1,5 +1,5 @@
 /*
-	SPDX-FileCopyrightText: 2011-2016 Graeme Gott <graeme@gottcode.org>
+	SPDX-FileCopyrightText: 2011-2024 Graeme Gott <graeme@gottcode.org>
 
 	SPDX-License-Identifier: GPL-3.0-or-later
 */
@@ -16,8 +16,8 @@ ToolBarList::ToolBarList(QWidget* parent)
 	m_toolbar = new QToolBar(this);
 	m_toolbar->setFloatable(false);
 	m_toolbar->setMovable(false);
-	m_toolbar->setToolButtonStyle(Qt::ToolButtonIconOnly);
-	m_toolbar->setStyleSheet("QToolBar { border-top: 1px solid palette(mid); }");
+	m_toolbar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
+	m_toolbar->setStyleSheet("QToolBar { border-bottom: 1px solid palette(mid); }");
 	m_toolbar->hide();
 
 	setContextMenuPolicy(Qt::ActionsContextMenu);
@@ -34,21 +34,20 @@ void ToolBarList::addToolBarAction(QAction* action)
 		addAction(action);
 		m_toolbar->addAction(action);
 		m_toolbar->show();
+		m_toolbar->widgetForAction(action)->setFocusPolicy(Qt::TabFocus);
 		updateGeometries();
 	}
 }
 
 //-----------------------------------------------------------------------------
 
-QIcon ToolBarList::fetchIcon(const QString& name)
+void ToolBarList::addToolBarWidget(QWidget* widget)
 {
-	QIcon icon(QString(":/tango/64x64/%1.png").arg(name));
-	icon.addFile(QString(":/tango/48x48/%1.png").arg(name));
-	icon.addFile(QString(":/tango/32x32/%1.png").arg(name));
-	icon.addFile(QString(":/tango/24x24/%1.png").arg(name));
-	icon.addFile(QString(":/tango/22x22/%1.png").arg(name));
-	icon.addFile(QString(":/tango/16x16/%1.png").arg(name));
-	return QIcon::fromTheme(name, icon);
+	if (widget) {
+		m_toolbar->addWidget(widget);
+		m_toolbar->show();
+		updateGeometries();
+	}
 }
 
 //-----------------------------------------------------------------------------
@@ -56,13 +55,12 @@ QIcon ToolBarList::fetchIcon(const QString& name)
 void ToolBarList::updateGeometries()
 {
 	// Resize viewport margins
-	QSize hint = m_toolbar->isHidden() ? QSize(0,0) : m_toolbar->sizeHint();
-	setViewportMargins(0, 0, 0, hint.height());
+	const QSize hint = m_toolbar->isHidden() ? QSize(0,0) : m_toolbar->sizeHint();
+	setViewportMargins(0, hint.height(), 0, 0);
 
 	// Resize toolbar
-	QRect rect = viewport()->geometry();
-	QRect geometry(rect.left(), rect.top() + rect.height(), rect.width(), hint.height());
-	m_toolbar->setGeometry(geometry);
+	const QRect rect = viewport()->geometry();
+	m_toolbar->setGeometry(rect.left(), rect.top() - hint.height(), rect.width(), hint.height());
 
 	QListWidget::updateGeometries();
 }
